# Copyright 2013 Cloudera Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import, print_function

import sys
import time

from impala.error import NotSupportedError

if sys.version_info[:2] <= (2, 6):
    import unittest2 as unittest
else:
    import unittest

import pytest

from impala.dbapi import connect, AUTH_MECHANISMS
from impala.util import _random_id
from impala.tests.util import ImpylaTestEnv, SocketTracker

from thrift.transport.TTransport import TTransportException

ENV = ImpylaTestEnv()
DEFAULT_AUTH = True
DEFAULT_AUTH_ERROR = "Non-default authorization method"

# JWT tests require the Impala coordinator to have the following startup flags:
# --jwt_token_auth=true --jwt_validate_signature=false --jwt_allow_without_tls=true
JWT_DISABLED = True
JWT_DISABLED_ERROR = "JWT authentication disabled"


# SSL can be enabled in Impala dev environment with the following commands:
# export IMPALA_SSL_CERT_DIR=$IMPALA_HOME/be/src/testutil
# export IMPALA_SSL_ARGS="--ssl_client_ca_certificate=$IMPALA_SSL_CERT_DIR/server-cert.pem --ssl_server_certificate=$IMPALA_SSL_CERT_DIR/server-cert.pem --ssl_private_key=$IMPALA_SSL_CERT_DIR/server-key.pem --hostname=localhost"
# bin/start-impala-cluster.py --impalad_args="$IMPALA_SSL_ARGS" --catalogd_args="$IMPALA_SSL_ARGS" --state_store_args="$IMPALA_SSL_ARGS"
# export IMPYLA_SSL_CERT=$IMPALA_SSL_CERT_DIR/server-cert.pem
SSL_DISABLED = ENV.ssl_cert == ""
SSL_DISABLED_ERROR = "No ssl certificate set."

# Table loading / DDLs can take several seconds in Impala. Use a larger timeout to avoid flaky tests.
TIMEOUT_S = 10

class ImpalaConnectionTests(unittest.TestCase):

    table_prefix = _random_id(prefix='dbapi20test_')
    tablename = table_prefix + 'contests'

    def setUp(self):
        self.connection = None

    def tearDown(self):
        if self.connection:
            self.connection.close()

    def _execute_queries(self, con):
        ddl = """
            CREATE TABLE {0} (
              f1 INT,
              f2 INT)
        """.format(self.tablename)
        try:
            cur = con.cursor()
            cur.execute(ddl)
            con.commit()
            cur.execute('DROP TABLE {0}'.format(self.tablename))
            con.commit()
        except:
            raise

    def _execute_query_get_username(self, con):
        query = "select user()"
        username_rows = None
        try:
            cur = con.cursor()
            cur.execute(query)
            username_rows = cur.fetchall()
        except:
            raise

        # Expecting the username as a single row with single column
        assert(username_rows is not None)
        assert(len(username_rows) == 1)
        assert(len(username_rows[0]) == 1)
        username = username_rows[0][0]
        return username

    def test_impala_nosasl_connect(self):
        self.connection = connect(ENV.host, ENV.port, timeout=TIMEOUT_S)
        self._execute_queries(self.connection)

    @pytest.mark.skipif(ENV.skip_hive_tests, reason="Skipping hive tests")
    def test_hive_plain_connect(self):
        self.connection = connect(ENV.host, ENV.hive_port,
                                  auth_mechanism="PLAIN",
                                  timeout=TIMEOUT_S,
                                  user=ENV.hive_user,
                                  password="cloudera")
        self._execute_queries(self.connection)

    @pytest.mark.skipif(DEFAULT_AUTH, reason=DEFAULT_AUTH_ERROR)
    def test_impala_plain_connect(self):
        self.connection = connect(ENV.host, ENV.port, auth_mechanism="PLAIN",
                                  timeout=TIMEOUT_S,
                                  user=ENV.hive_user,
                                  password="cloudera")
        self._execute_queries(self.connection)

    @pytest.mark.skipif(DEFAULT_AUTH, reason=DEFAULT_AUTH_ERROR)
    def test_impala_ldap_connect_user_agent(self):
        self.connection = connect(ENV.host, ENV.port, auth_mechanism="LDAP",
                                  timeout=TIMEOUT_S,
                                  user=ENV.hive_user,
                                  password="cloudera",
                                  http_path="http-path",
                                  use_http_transport=True,
                                  use_ssl=True,
                                  user_agent="cloudera/impyla")
        self._execute_queries(self.connection)

    @pytest.mark.skipif(DEFAULT_AUTH, reason=DEFAULT_AUTH_ERROR)
    def test_hive_nosasl_connect(self):
        self.connection = connect(ENV.host, ENV.hive_port, timeout=TIMEOUT_S)
        self._execute_queries(self.connection)

    @pytest.mark.params_neg
    def test_bad_auth(self):
        """Test some simple error messages"""
        try:
            connect(ENV.host, ENV.port, auth_mechanism="foo")
            assert False, "'connect' method should have thrown an exception but did not"
        except NotSupportedError as e:
            assert 'Unsupported authentication mechanism: FOO' in str(e)

    @pytest.mark.skipif(JWT_DISABLED, reason=JWT_DISABLED_ERROR)
    def test_jwt_auth(self):
        """Test for connecting via the auth_mechanism=JWT"""
        # This is a JWT generated via jwt.io's online generator
        # The content is:
        # Header:
        # {
        #   "alg": "HS256",
        #   "typ": "JWT"
        # }
        # Payload:
        # {
        #   "sub": "1234567890",
        #   "username": "impylajwttest",
        #   "iat": 1516239022
        # }
        # Signature by key "impylaimpylaimpylaimpylaimpylaimp"
        jwt = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiIxMjM0NTY3ODkwIiwidXNlcm5hbWUiOiJpbXB5bGFqd3R0ZXN0IiwiaWF0IjoxNTE2MjM5MDIyfQ.Uvq2TZ9nRtM-JgFEd_fL2Z0kFcflx0Tr5cbJR4yySyc"
        self.connection = connect(ENV.host, ENV.http_port, use_http_transport=True,
                                  http_path="cliservice", auth_mechanism="JWT",
                                  timeout=TIMEOUT_S, jwt=jwt)
        username = self._execute_query_get_username(self.connection)
        assert(username == "impylajwttest")
        print("Username: {0}".format(username))
        self._execute_queries(self.connection)

    @pytest.mark.params_neg
    def test_jwt_auth_missing_jwt(self):
        """Test for the expected error when JWT auth is used and a JWT is not provided"""
        try:
            connect(use_http_transport=True, auth_mechanism="JWT")
            assert False, "'connect' method should have thrown an exception but did not"
        except NotSupportedError as e:
            assert "JWT authentication requires specifying the 'jwt' argument" in str(e)

    @pytest.mark.params_neg
    def test_jwt_auth_invalid_transport(self):
        """Test for the expected error when JWT auth is used without the HTTP transport"""
        try:
            connect(auth_mechanism="JWT", jwt="dummy.jwt.arg")
            assert False, "'connect' method should have thrown an exception but did not"
        except NotSupportedError as e:
            assert "JWT authentication is only supported for HTTP transport" in str(e)

    @pytest.mark.params_neg
    def test_non_jwt_auth_with_jwt(self):
        """Test for the expected error when authentication is anything other than JWT and the 'jwt' parameter is specified"""
        def run_test(auth_mechanism):
            try:
                connect(auth_mechanism=auth_mechanism, jwt="dummy.jwt.arg")
                assert False, "'connect' method should have thrown an exception but did not"
            except NotSupportedError as e:
                assert "'jwt' argument cannot be specified with '{0}' authentication".format(auth_mechanism) in str(e)

        for auth_mech in AUTH_MECHANISMS:
            if auth_mech is not "JWT":
                run_test(auth_mech)

    @pytest.mark.params_neg
    def test_jwt_auth_with_user(self):
        """Test for the expected error when authentication is JWT and the 'user' parameter is specified"""
        try:
            connect(auth_mechanism="JWT", jwt="dummy.jwt.arg", use_http_transport=True, user="any_user")
            assert False, "'connect' method should have thrown an exception but did not"
        except NotSupportedError as e:
            assert "'user' argument cannot be specified with 'JWT' authentication" in str(e)

    @pytest.mark.params_neg
    def test_jwt_auth_with_ldap_user(self):
        """Test for the expected error when authentication is JWT and the 'ldap_user' parameter is specified"""
        try:
            connect(auth_mechanism="JWT", jwt="dummy.jwt.arg", use_http_transport=True, ldap_user="any_user")
            assert False, "'connect' method should have thrown an exception but did not"
        except NotSupportedError as e:
            assert "'user' argument cannot be specified with 'JWT' authentication" in str(e)

    @pytest.mark.params_neg
    def test_jwt_auth_with_password(self):
        """Test for the expected error when authentication is JWT and the 'password' parameter is specified"""
        try:
            connect(auth_mechanism="JWT", jwt="dummy.jwt.arg", use_http_transport=True, password="any_password")
            assert False, "'connect' method should have thrown an exception but did not"
        except NotSupportedError as e:
            assert "'password' argument cannot be specified with 'JWT' authentication" in str(e)

    @pytest.mark.params_neg
    def test_jwt_auth_with_ldap_password(self):
        """Test for the expected error when authentication is JWT and the 'ldap_password' parameter is specified"""
        try:
            connect(auth_mechanism="JWT", jwt="dummy.jwt.arg", use_http_transport=True, ldap_password="any_password")
            assert False, "'connect' method should have thrown an exception but did not"
        except NotSupportedError as e:
            assert "'password' argument cannot be specified with 'JWT' authentication" in str(e)

    @pytest.mark.skipif(SSL_DISABLED, reason=SSL_DISABLED_ERROR)
    def test_ssl_connection_no_cert(self):
        self.connection = connect(ENV.host, ENV.port, timeout=TIMEOUT_S, use_ssl=True)
        self._execute_queries(self.connection)


    @pytest.mark.skipif(SSL_DISABLED, reason=SSL_DISABLED_ERROR)
    def test_ssl_connection_with_cert(self):
        self.connection = connect(
            ENV.host, ENV.port, use_ssl=True, timeout=TIMEOUT_S, ca_cert=ENV.ssl_cert)
        self._execute_queries(self.connection)

    @pytest.mark.skipif(SSL_DISABLED, reason=SSL_DISABLED_ERROR)
    def test_https_connection(self):
        self.connection = connect(ENV.host, ENV.http_port, use_http_transport=True,
                                  http_path="cliservice", use_ssl=True, timeout=TIMEOUT_S)
        self._execute_queries(self.connection)

    def test_retry_dml(self):
        """Regression test for #549.
        Checks the INSERT statements are not retried when the client disconnects due to timeout
        during execute()."""
        # Connection with higher timeout to avoid errors.
        self.connection = connect(ENV.host, ENV.port, timeout=TIMEOUT_S)
        create = "CREATE TABLE {0} (f1 INT)".format(self.tablename)
        refresh = "REFRESH {0}".format(self.tablename)
        insert = "INSERT INTO TABLE {0} SELECT 1".format(self.tablename)
        select = "SELECT * FROM {0}".format(self.tablename)
        drop = "DROP TABLE {0}".format(self.tablename)
        reliable_cur = self.connection.cursor()
        reliable_cur.execute(create)
        reliable_cur.execute(refresh) # Load table metadata to make subsequent operations faster.
        successful_inserts = 0
        NUM_INSERTS = 3
        for i in range(NUM_INSERTS):
            # Connect with low timeout to trigger failed RPCs.
            LOW_TIMEOUT_S = 1
            low_timeout_connection = connect(ENV.host, ENV.port, timeout=LOW_TIMEOUT_S)
            low_timeout_cur = low_timeout_connection.cursor()
            try:
                # Use IMPALAD_LOAD_TABLES_DELAY>LOW_TIMEOUT_S to trigger timeout.
                # IMPALAD_LOAD_TABLES_DELAY was added in Impala 4.4.0 (IMPALA-12493), so with
                # older Impala servers the test is not expected to work.
                DELAY_S = LOW_TIMEOUT_S + 1
                configuration = {"debug_action": "IMPALAD_LOAD_TABLES_DELAY:SLEEP@{}".format(1000 * DELAY_S)}
                low_timeout_cur.execute(insert, configuration=configuration)
                successful_inserts += 1
            except TTransportException:
                # Sleep until the insert is expected to be finished and close the session.
                SLEEP_S = DELAY_S - LOW_TIMEOUT_S + 2
                time.sleep(SLEEP_S)
                # Reopen the transport to allow closing the session correctly.
                low_timeout_cur.session.client._iprot.trans.close()
                low_timeout_cur.session.client._iprot.trans.open()
                low_timeout_cur.close()
                low_timeout_connection.close()
        assert successful_inserts == 0
        # Use the reliable connection for result checking and cleanup.
        reliable_cur.execute(select)
        result = reliable_cur.fetchall()
        # All inserts must have actually finished after client timeout.
        assert len(result) == NUM_INSERTS
        reliable_cur.execute(drop)
        # If all inserts are successful then probably the Impala cluster is too fast.
        assert successful_inserts < NUM_INSERTS


class ImpalaSocketTests(unittest.TestCase):

    def run_a_query(self):
        with connect(ENV.host, ENV.port) as connection:
            with connection.cursor() as cursor:
                cursor.execute('select 1 as a_number')
                return cursor.fetchall()

    def test_socket_leak(self):
        with SocketTracker() as sockets:
            # there should be no open sockets prior to running query
            assert len(sockets.open_sockets) == 0, 'Expected 0 open sockets, but saw {}'.format(
                sockets.open_sockets)
            # run a query!
            self.run_a_query()
            # the "run_query" method should have caused all sockets to close
            assert len(sockets.open_sockets) == 0, 'Expected 0 open sockets, but saw {}'.format(
                sockets.open_sockets)



#
# Autogenerated by Thrift Compiler (0.16.0)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py:new_style,no_utf8strings
#

from thrift.Thrift import TType, TMessageType, TFrozenDict, TException, TApplicationException
from thrift.protocol.TProtocol import TProtocolException
from thrift.TRecursive import fix_spec


from thrift.transport import TTransport
all_structs = []


class TErrorCode(object):
    OK = 0
    UNUSED = 1
    GENERAL = 2
    CANCELLED = 3
    ANALYSIS_ERROR = 4
    NOT_IMPLEMENTED_ERROR = 5
    RUNTIME_ERROR = 6
    MEM_LIMIT_EXCEEDED = 7
    INTERNAL_ERROR = 8
    RECOVERABLE_ERROR = 9
    PARQUET_MULTIPLE_BLOCKS = 10
    PARQUET_COLUMN_METADATA_INVALID = 11
    PARQUET_HEADER_PAGE_SIZE_EXCEEDED = 12
    PARQUET_HEADER_EOF = 13
    PARQUET_GROUP_ROW_COUNT_ERROR = 14
    PARQUET_GROUP_ROW_COUNT_OVERFLOW = 15
    PARQUET_MISSING_PRECISION = 16
    PARQUET_WRONG_PRECISION = 17
    PARQUET_BAD_CONVERTED_TYPE = 18
    PARQUET_INCOMPATIBLE_DECIMAL = 19
    SEQUENCE_SCANNER_PARSE_ERROR = 20
    SNAPPY_DECOMPRESS_INVALID_BLOCK_SIZE = 21
    SNAPPY_DECOMPRESS_INVALID_COMPRESSED_LENGTH = 22
    SNAPPY_DECOMPRESS_UNCOMPRESSED_LENGTH_FAILED = 23
    SNAPPY_DECOMPRESS_RAW_UNCOMPRESS_FAILED = 24
    SNAPPY_DECOMPRESS_DECOMPRESS_SIZE_INCORRECT = 25
    FRAGMENT_EXECUTOR = 26
    PARTITIONED_HASH_JOIN_MAX_PARTITION_DEPTH = 27
    PARTITIONED_AGG_MAX_PARTITION_DEPTH = 28
    MISSING_BUILTIN = 29
    RPC_GENERAL_ERROR = 30
    RPC_RECV_TIMEOUT = 31
    UDF_VERIFY_FAILED = 32
    PARQUET_CORRUPT_RLE_BYTES = 33
    AVRO_DECIMAL_RESOLUTION_ERROR = 34
    AVRO_DECIMAL_METADATA_MISMATCH = 35
    AVRO_SCHEMA_RESOLUTION_ERROR = 36
    AVRO_SCHEMA_METADATA_MISMATCH = 37
    AVRO_UNSUPPORTED_DEFAULT_VALUE = 38
    AVRO_MISSING_FIELD = 39
    AVRO_MISSING_DEFAULT = 40
    AVRO_NULLABILITY_MISMATCH = 41
    AVRO_NOT_A_RECORD = 42
    PARQUET_DEF_LEVEL_ERROR = 43
    PARQUET_NUM_COL_VALS_ERROR = 44
    PARQUET_DICT_DECODE_FAILURE = 45
    SSL_PASSWORD_CMD_FAILED = 46
    SSL_CERTIFICATE_PATH_BLANK = 47
    SSL_PRIVATE_KEY_PATH_BLANK = 48
    SSL_CERTIFICATE_NOT_FOUND = 49
    SSL_PRIVATE_KEY_NOT_FOUND = 50
    SSL_SOCKET_CREATION_FAILED = 51
    MEM_ALLOC_FAILED = 52
    PARQUET_REP_LEVEL_ERROR = 53
    PARQUET_UNRECOGNIZED_SCHEMA = 54
    COLLECTION_ALLOC_FAILED = 55
    TMP_DEVICE_BLACKLISTED = 56
    TMP_FILE_BLACKLISTED = 57
    RPC_CLIENT_CONNECT_FAILURE = 58
    STALE_METADATA_FILE_TOO_SHORT = 59
    PARQUET_BAD_VERSION_NUMBER = 60
    SCANNER_INCOMPLETE_READ = 61
    SCANNER_INVALID_READ = 62
    AVRO_BAD_VERSION_HEADER = 63
    UDF_MEM_LIMIT_EXCEEDED = 64
    UNUSED_65 = 65
    COMPRESSED_FILE_MULTIPLE_BLOCKS = 66
    COMPRESSED_FILE_BLOCK_CORRUPTED = 67
    COMPRESSED_FILE_DECOMPRESSOR_ERROR = 68
    COMPRESSED_FILE_DECOMPRESSOR_NO_PROGRESS = 69
    COMPRESSED_FILE_TRUNCATED = 70
    DATASTREAM_SENDER_TIMEOUT = 71
    KUDU_IMPALA_TYPE_MISSING = 72
    IMPALA_KUDU_TYPE_MISSING = 73
    KUDU_NOT_SUPPORTED_ON_OS = 74
    KUDU_NOT_ENABLED = 75
    PARTITIONED_HASH_JOIN_REPARTITION_FAILS = 76
    UNUSED_77 = 77
    AVRO_TRUNCATED_BLOCK = 78
    AVRO_INVALID_UNION = 79
    AVRO_INVALID_BOOLEAN = 80
    AVRO_INVALID_LENGTH = 81
    SCANNER_INVALID_INT = 82
    AVRO_INVALID_RECORD_COUNT = 83
    AVRO_INVALID_COMPRESSED_SIZE = 84
    AVRO_INVALID_METADATA_COUNT = 85
    SCANNER_STRING_LENGTH_OVERFLOW = 86
    PARQUET_CORRUPT_PLAIN_VALUE = 87
    PARQUET_CORRUPT_DICTIONARY = 88
    TEXT_PARSER_TRUNCATED_COLUMN = 89
    SCRATCH_LIMIT_EXCEEDED = 90
    BUFFER_ALLOCATION_FAILED = 91
    PARQUET_ZERO_ROWS_IN_NON_EMPTY_FILE = 92
    NO_REGISTERED_BACKENDS = 93
    KUDU_KEY_ALREADY_PRESENT = 94
    KUDU_NOT_FOUND = 95
    KUDU_SESSION_ERROR = 96
    AVRO_UNSUPPORTED_TYPE = 97
    AVRO_INVALID_DECIMAL = 98
    KUDU_NULL_CONSTRAINT_VIOLATION = 99
    PARQUET_TIMESTAMP_OUT_OF_RANGE = 100
    SCRATCH_ALLOCATION_FAILED = 101
    SCRATCH_READ_TRUNCATED = 102
    KUDU_TIMESTAMP_OUT_OF_RANGE = 103
    MAX_ROW_SIZE = 104
    IR_VERIFY_FAILED = 105
    MINIMUM_RESERVATION_UNAVAILABLE = 106
    ADMISSION_REJECTED = 107
    ADMISSION_TIMED_OUT = 108
    THREAD_CREATION_FAILED = 109
    DISK_IO_ERROR = 110
    DATASTREAM_RECVR_CLOSED = 111
    BAD_PRINCIPAL_FORMAT = 112
    LZ4_COMPRESSION_INPUT_TOO_LARGE = 113
    SASL_APP_NAME_MISMATCH = 114
    PARQUET_BIT_PACKED_LEVELS = 115
    ROW_BATCH_TOO_LARGE = 116
    LIB_VERSION_MISMATCH = 117
    SCRATCH_READ_VERIFY_FAILED = 118
    CANCELLED_INTERNALLY = 119
    SERVER_SHUTTING_DOWN = 120
    PARQUET_TIMESTAMP_INVALID_TIME_OF_DAY = 121
    PARQUET_CORRUPT_BOOL_VALUE = 122
    THREAD_POOL_SUBMIT_FAILED = 123
    THREAD_POOL_TASK_TIMED_OUT = 124
    UNREACHABLE_IMPALADS = 125
    INACTIVE_SESSION_EXPIRED = 126
    INACTIVE_QUERY_EXPIRED = 127
    EXEC_TIME_LIMIT_EXCEEDED = 128
    CPU_LIMIT_EXCEEDED = 129
    SCAN_BYTES_LIMIT_EXCEEDED = 130
    ROWS_PRODUCED_LIMIT_EXCEEDED = 131
    EXPR_REWRITE_RESULT_LIMIT_EXCEEDED = 132
    UNRESPONSIVE_BACKEND = 133
    PARQUET_DATE_OUT_OF_RANGE = 134
    DISCONNECTED_SESSION_CLOSED = 135
    UNAUTHORIZED_SESSION_USER = 136
    ZSTD_ERROR = 137
    LZ4_BLOCK_DECOMPRESS_DECOMPRESS_SIZE_INCORRECT = 138
    LZ4_BLOCK_DECOMPRESS_INVALID_INPUT_LENGTH = 139
    LZ4_BLOCK_DECOMPRESS_INVALID_COMPRESSED_LENGTH = 140
    LZ4_DECOMPRESS_SAFE_FAILED = 141
    LZ4_COMPRESS_DEFAULT_FAILED = 142
    MAX_STATEMENT_LENGTH_EXCEEDED = 143
    AVRO_INVALID_DATE = 144
    ORC_TIMESTAMP_OUT_OF_RANGE = 145
    ORC_DATE_OUT_OF_RANGE = 146
    ORC_NESTED_TYPE_MISMATCH = 147
    ORC_TYPE_NOT_ROOT_AT_STRUCT = 148
    NAAJ_OUT_OF_MEMORY = 149
    INVALID_QUERY_HANDLE = 150
    JOIN_ROWS_PRODUCED_LIMIT_EXCEEDED = 151
    LOCAL_DISK_FAULTY = 152
    JWKS_PARSE_ERROR = 153
    JWT_VERIFY_FAILED = 154
    PARQUET_ROWS_SKIPPING = 155
    QUERY_OPTION_PARSE_FAILED = 156
    CATALOG_INCOMPATIBLE_PROTOCOL = 157
    STATESTORE_INCOMPATIBLE_PROTOCOL = 158
    JDBC_CONFIGURATION_ERROR = 159
    TUPLE_CACHE_INCONSISTENCY = 160
    OAUTH_VERIFY_FAILED = 161

    _VALUES_TO_NAMES = {
        0: "OK",
        1: "UNUSED",
        2: "GENERAL",
        3: "CANCELLED",
        4: "ANALYSIS_ERROR",
        5: "NOT_IMPLEMENTED_ERROR",
        6: "RUNTIME_ERROR",
        7: "MEM_LIMIT_EXCEEDED",
        8: "INTERNAL_ERROR",
        9: "RECOVERABLE_ERROR",
        10: "PARQUET_MULTIPLE_BLOCKS",
        11: "PARQUET_COLUMN_METADATA_INVALID",
        12: "PARQUET_HEADER_PAGE_SIZE_EXCEEDED",
        13: "PARQUET_HEADER_EOF",
        14: "PARQUET_GROUP_ROW_COUNT_ERROR",
        15: "PARQUET_GROUP_ROW_COUNT_OVERFLOW",
        16: "PARQUET_MISSING_PRECISION",
        17: "PARQUET_WRONG_PRECISION",
        18: "PARQUET_BAD_CONVERTED_TYPE",
        19: "PARQUET_INCOMPATIBLE_DECIMAL",
        20: "SEQUENCE_SCANNER_PARSE_ERROR",
        21: "SNAPPY_DECOMPRESS_INVALID_BLOCK_SIZE",
        22: "SNAPPY_DECOMPRESS_INVALID_COMPRESSED_LENGTH",
        23: "SNAPPY_DECOMPRESS_UNCOMPRESSED_LENGTH_FAILED",
        24: "SNAPPY_DECOMPRESS_RAW_UNCOMPRESS_FAILED",
        25: "SNAPPY_DECOMPRESS_DECOMPRESS_SIZE_INCORRECT",
        26: "FRAGMENT_EXECUTOR",
        27: "PARTITIONED_HASH_JOIN_MAX_PARTITION_DEPTH",
        28: "PARTITIONED_AGG_MAX_PARTITION_DEPTH",
        29: "MISSING_BUILTIN",
        30: "RPC_GENERAL_ERROR",
        31: "RPC_RECV_TIMEOUT",
        32: "UDF_VERIFY_FAILED",
        33: "PARQUET_CORRUPT_RLE_BYTES",
        34: "AVRO_DECIMAL_RESOLUTION_ERROR",
        35: "AVRO_DECIMAL_METADATA_MISMATCH",
        36: "AVRO_SCHEMA_RESOLUTION_ERROR",
        37: "AVRO_SCHEMA_METADATA_MISMATCH",
        38: "AVRO_UNSUPPORTED_DEFAULT_VALUE",
        39: "AVRO_MISSING_FIELD",
        40: "AVRO_MISSING_DEFAULT",
        41: "AVRO_NULLABILITY_MISMATCH",
        42: "AVRO_NOT_A_RECORD",
        43: "PARQUET_DEF_LEVEL_ERROR",
        44: "PARQUET_NUM_COL_VALS_ERROR",
        45: "PARQUET_DICT_DECODE_FAILURE",
        46: "SSL_PASSWORD_CMD_FAILED",
        47: "SSL_CERTIFICATE_PATH_BLANK",
        48: "SSL_PRIVATE_KEY_PATH_BLANK",
        49: "SSL_CERTIFICATE_NOT_FOUND",
        50: "SSL_PRIVATE_KEY_NOT_FOUND",
        51: "SSL_SOCKET_CREATION_FAILED",
        52: "MEM_ALLOC_FAILED",
        53: "PARQUET_REP_LEVEL_ERROR",
        54: "PARQUET_UNRECOGNIZED_SCHEMA",
        55: "COLLECTION_ALLOC_FAILED",
        56: "TMP_DEVICE_BLACKLISTED",
        57: "TMP_FILE_BLACKLISTED",
        58: "RPC_CLIENT_CONNECT_FAILURE",
        59: "STALE_METADATA_FILE_TOO_SHORT",
        60: "PARQUET_BAD_VERSION_NUMBER",
        61: "SCANNER_INCOMPLETE_READ",
        62: "SCANNER_INVALID_READ",
        63: "AVRO_BAD_VERSION_HEADER",
        64: "UDF_MEM_LIMIT_EXCEEDED",
        65: "UNUSED_65",
        66: "COMPRESSED_FILE_MULTIPLE_BLOCKS",
        67: "COMPRESSED_FILE_BLOCK_CORRUPTED",
        68: "COMPRESSED_FILE_DECOMPRESSOR_ERROR",
        69: "COMPRESSED_FILE_DECOMPRESSOR_NO_PROGRESS",
        70: "COMPRESSED_FILE_TRUNCATED",
        71: "DATASTREAM_SENDER_TIMEOUT",
        72: "KUDU_IMPALA_TYPE_MISSING",
        73: "IMPALA_KUDU_TYPE_MISSING",
        74: "KUDU_NOT_SUPPORTED_ON_OS",
        75: "KUDU_NOT_ENABLED",
        76: "PARTITIONED_HASH_JOIN_REPARTITION_FAILS",
        77: "UNUSED_77",
        78: "AVRO_TRUNCATED_BLOCK",
        79: "AVRO_INVALID_UNION",
        80: "AVRO_INVALID_BOOLEAN",
        81: "AVRO_INVALID_LENGTH",
        82: "SCANNER_INVALID_INT",
        83: "AVRO_INVALID_RECORD_COUNT",
        84: "AVRO_INVALID_COMPRESSED_SIZE",
        85: "AVRO_INVALID_METADATA_COUNT",
        86: "SCANNER_STRING_LENGTH_OVERFLOW",
        87: "PARQUET_CORRUPT_PLAIN_VALUE",
        88: "PARQUET_CORRUPT_DICTIONARY",
        89: "TEXT_PARSER_TRUNCATED_COLUMN",
        90: "SCRATCH_LIMIT_EXCEEDED",
        91: "BUFFER_ALLOCATION_FAILED",
        92: "PARQUET_ZERO_ROWS_IN_NON_EMPTY_FILE",
        93: "NO_REGISTERED_BACKENDS",
        94: "KUDU_KEY_ALREADY_PRESENT",
        95: "KUDU_NOT_FOUND",
        96: "KUDU_SESSION_ERROR",
        97: "AVRO_UNSUPPORTED_TYPE",
        98: "AVRO_INVALID_DECIMAL",
        99: "KUDU_NULL_CONSTRAINT_VIOLATION",
        100: "PARQUET_TIMESTAMP_OUT_OF_RANGE",
        101: "SCRATCH_ALLOCATION_FAILED",
        102: "SCRATCH_READ_TRUNCATED",
        103: "KUDU_TIMESTAMP_OUT_OF_RANGE",
        104: "MAX_ROW_SIZE",
        105: "IR_VERIFY_FAILED",
        106: "MINIMUM_RESERVATION_UNAVAILABLE",
        107: "ADMISSION_REJECTED",
        108: "ADMISSION_TIMED_OUT",
        109: "THREAD_CREATION_FAILED",
        110: "DISK_IO_ERROR",
        111: "DATASTREAM_RECVR_CLOSED",
        112: "BAD_PRINCIPAL_FORMAT",
        113: "LZ4_COMPRESSION_INPUT_TOO_LARGE",
        114: "SASL_APP_NAME_MISMATCH",
        115: "PARQUET_BIT_PACKED_LEVELS",
        116: "ROW_BATCH_TOO_LARGE",
        117: "LIB_VERSION_MISMATCH",
        118: "SCRATCH_READ_VERIFY_FAILED",
        119: "CANCELLED_INTERNALLY",
        120: "SERVER_SHUTTING_DOWN",
        121: "PARQUET_TIMESTAMP_INVALID_TIME_OF_DAY",
        122: "PARQUET_CORRUPT_BOOL_VALUE",
        123: "THREAD_POOL_SUBMIT_FAILED",
        124: "THREAD_POOL_TASK_TIMED_OUT",
        125: "UNREACHABLE_IMPALADS",
        126: "INACTIVE_SESSION_EXPIRED",
        127: "INACTIVE_QUERY_EXPIRED",
        128: "EXEC_TIME_LIMIT_EXCEEDED",
        129: "CPU_LIMIT_EXCEEDED",
        130: "SCAN_BYTES_LIMIT_EXCEEDED",
        131: "ROWS_PRODUCED_LIMIT_EXCEEDED",
        132: "EXPR_REWRITE_RESULT_LIMIT_EXCEEDED",
        133: "UNRESPONSIVE_BACKEND",
        134: "PARQUET_DATE_OUT_OF_RANGE",
        135: "DISCONNECTED_SESSION_CLOSED",
        136: "UNAUTHORIZED_SESSION_USER",
        137: "ZSTD_ERROR",
        138: "LZ4_BLOCK_DECOMPRESS_DECOMPRESS_SIZE_INCORRECT",
        139: "LZ4_BLOCK_DECOMPRESS_INVALID_INPUT_LENGTH",
        140: "LZ4_BLOCK_DECOMPRESS_INVALID_COMPRESSED_LENGTH",
        141: "LZ4_DECOMPRESS_SAFE_FAILED",
        142: "LZ4_COMPRESS_DEFAULT_FAILED",
        143: "MAX_STATEMENT_LENGTH_EXCEEDED",
        144: "AVRO_INVALID_DATE",
        145: "ORC_TIMESTAMP_OUT_OF_RANGE",
        146: "ORC_DATE_OUT_OF_RANGE",
        147: "ORC_NESTED_TYPE_MISMATCH",
        148: "ORC_TYPE_NOT_ROOT_AT_STRUCT",
        149: "NAAJ_OUT_OF_MEMORY",
        150: "INVALID_QUERY_HANDLE",
        151: "JOIN_ROWS_PRODUCED_LIMIT_EXCEEDED",
        152: "LOCAL_DISK_FAULTY",
        153: "JWKS_PARSE_ERROR",
        154: "JWT_VERIFY_FAILED",
        155: "PARQUET_ROWS_SKIPPING",
        156: "QUERY_OPTION_PARSE_FAILED",
        157: "CATALOG_INCOMPATIBLE_PROTOCOL",
        158: "STATESTORE_INCOMPATIBLE_PROTOCOL",
        159: "JDBC_CONFIGURATION_ERROR",
        160: "TUPLE_CACHE_INCONSISTENCY",
        161: "OAUTH_VERIFY_FAILED",
    }

    _NAMES_TO_VALUES = {
        "OK": 0,
        "UNUSED": 1,
        "GENERAL": 2,
        "CANCELLED": 3,
        "ANALYSIS_ERROR": 4,
        "NOT_IMPLEMENTED_ERROR": 5,
        "RUNTIME_ERROR": 6,
        "MEM_LIMIT_EXCEEDED": 7,
        "INTERNAL_ERROR": 8,
        "RECOVERABLE_ERROR": 9,
        "PARQUET_MULTIPLE_BLOCKS": 10,
        "PARQUET_COLUMN_METADATA_INVALID": 11,
        "PARQUET_HEADER_PAGE_SIZE_EXCEEDED": 12,
        "PARQUET_HEADER_EOF": 13,
        "PARQUET_GROUP_ROW_COUNT_ERROR": 14,
        "PARQUET_GROUP_ROW_COUNT_OVERFLOW": 15,
        "PARQUET_MISSING_PRECISION": 16,
        "PARQUET_WRONG_PRECISION": 17,
        "PARQUET_BAD_CONVERTED_TYPE": 18,
        "PARQUET_INCOMPATIBLE_DECIMAL": 19,
        "SEQUENCE_SCANNER_PARSE_ERROR": 20,
        "SNAPPY_DECOMPRESS_INVALID_BLOCK_SIZE": 21,
        "SNAPPY_DECOMPRESS_INVALID_COMPRESSED_LENGTH": 22,
        "SNAPPY_DECOMPRESS_UNCOMPRESSED_LENGTH_FAILED": 23,
        "SNAPPY_DECOMPRESS_RAW_UNCOMPRESS_FAILED": 24,
        "SNAPPY_DECOMPRESS_DECOMPRESS_SIZE_INCORRECT": 25,
        "FRAGMENT_EXECUTOR": 26,
        "PARTITIONED_HASH_JOIN_MAX_PARTITION_DEPTH": 27,
        "PARTITIONED_AGG_MAX_PARTITION_DEPTH": 28,
        "MISSING_BUILTIN": 29,
        "RPC_GENERAL_ERROR": 30,
        "RPC_RECV_TIMEOUT": 31,
        "UDF_VERIFY_FAILED": 32,
        "PARQUET_CORRUPT_RLE_BYTES": 33,
        "AVRO_DECIMAL_RESOLUTION_ERROR": 34,
        "AVRO_DECIMAL_METADATA_MISMATCH": 35,
        "AVRO_SCHEMA_RESOLUTION_ERROR": 36,
        "AVRO_SCHEMA_METADATA_MISMATCH": 37,
        "AVRO_UNSUPPORTED_DEFAULT_VALUE": 38,
        "AVRO_MISSING_FIELD": 39,
        "AVRO_MISSING_DEFAULT": 40,
        "AVRO_NULLABILITY_MISMATCH": 41,
        "AVRO_NOT_A_RECORD": 42,
        "PARQUET_DEF_LEVEL_ERROR": 43,
        "PARQUET_NUM_COL_VALS_ERROR": 44,
        "PARQUET_DICT_DECODE_FAILURE": 45,
        "SSL_PASSWORD_CMD_FAILED": 46,
        "SSL_CERTIFICATE_PATH_BLANK": 47,
        "SSL_PRIVATE_KEY_PATH_BLANK": 48,
        "SSL_CERTIFICATE_NOT_FOUND": 49,
        "SSL_PRIVATE_KEY_NOT_FOUND": 50,
        "SSL_SOCKET_CREATION_FAILED": 51,
        "MEM_ALLOC_FAILED": 52,
        "PARQUET_REP_LEVEL_ERROR": 53,
        "PARQUET_UNRECOGNIZED_SCHEMA": 54,
        "COLLECTION_ALLOC_FAILED": 55,
        "TMP_DEVICE_BLACKLISTED": 56,
        "TMP_FILE_BLACKLISTED": 57,
        "RPC_CLIENT_CONNECT_FAILURE": 58,
        "STALE_METADATA_FILE_TOO_SHORT": 59,
        "PARQUET_BAD_VERSION_NUMBER": 60,
        "SCANNER_INCOMPLETE_READ": 61,
        "SCANNER_INVALID_READ": 62,
        "AVRO_BAD_VERSION_HEADER": 63,
        "UDF_MEM_LIMIT_EXCEEDED": 64,
        "UNUSED_65": 65,
        "COMPRESSED_FILE_MULTIPLE_BLOCKS": 66,
        "COMPRESSED_FILE_BLOCK_CORRUPTED": 67,
        "COMPRESSED_FILE_DECOMPRESSOR_ERROR": 68,
        "COMPRESSED_FILE_DECOMPRESSOR_NO_PROGRESS": 69,
        "COMPRESSED_FILE_TRUNCATED": 70,
        "DATASTREAM_SENDER_TIMEOUT": 71,
        "KUDU_IMPALA_TYPE_MISSING": 72,
        "IMPALA_KUDU_TYPE_MISSING": 73,
        "KUDU_NOT_SUPPORTED_ON_OS": 74,
        "KUDU_NOT_ENABLED": 75,
        "PARTITIONED_HASH_JOIN_REPARTITION_FAILS": 76,
        "UNUSED_77": 77,
        "AVRO_TRUNCATED_BLOCK": 78,
        "AVRO_INVALID_UNION": 79,
        "AVRO_INVALID_BOOLEAN": 80,
        "AVRO_INVALID_LENGTH": 81,
        "SCANNER_INVALID_INT": 82,
        "AVRO_INVALID_RECORD_COUNT": 83,
        "AVRO_INVALID_COMPRESSED_SIZE": 84,
        "AVRO_INVALID_METADATA_COUNT": 85,
        "SCANNER_STRING_LENGTH_OVERFLOW": 86,
        "PARQUET_CORRUPT_PLAIN_VALUE": 87,
        "PARQUET_CORRUPT_DICTIONARY": 88,
        "TEXT_PARSER_TRUNCATED_COLUMN": 89,
        "SCRATCH_LIMIT_EXCEEDED": 90,
        "BUFFER_ALLOCATION_FAILED": 91,
        "PARQUET_ZERO_ROWS_IN_NON_EMPTY_FILE": 92,
        "NO_REGISTERED_BACKENDS": 93,
        "KUDU_KEY_ALREADY_PRESENT": 94,
        "KUDU_NOT_FOUND": 95,
        "KUDU_SESSION_ERROR": 96,
        "AVRO_UNSUPPORTED_TYPE": 97,
        "AVRO_INVALID_DECIMAL": 98,
        "KUDU_NULL_CONSTRAINT_VIOLATION": 99,
        "PARQUET_TIMESTAMP_OUT_OF_RANGE": 100,
        "SCRATCH_ALLOCATION_FAILED": 101,
        "SCRATCH_READ_TRUNCATED": 102,
        "KUDU_TIMESTAMP_OUT_OF_RANGE": 103,
        "MAX_ROW_SIZE": 104,
        "IR_VERIFY_FAILED": 105,
        "MINIMUM_RESERVATION_UNAVAILABLE": 106,
        "ADMISSION_REJECTED": 107,
        "ADMISSION_TIMED_OUT": 108,
        "THREAD_CREATION_FAILED": 109,
        "DISK_IO_ERROR": 110,
        "DATASTREAM_RECVR_CLOSED": 111,
        "BAD_PRINCIPAL_FORMAT": 112,
        "LZ4_COMPRESSION_INPUT_TOO_LARGE": 113,
        "SASL_APP_NAME_MISMATCH": 114,
        "PARQUET_BIT_PACKED_LEVELS": 115,
        "ROW_BATCH_TOO_LARGE": 116,
        "LIB_VERSION_MISMATCH": 117,
        "SCRATCH_READ_VERIFY_FAILED": 118,
        "CANCELLED_INTERNALLY": 119,
        "SERVER_SHUTTING_DOWN": 120,
        "PARQUET_TIMESTAMP_INVALID_TIME_OF_DAY": 121,
        "PARQUET_CORRUPT_BOOL_VALUE": 122,
        "THREAD_POOL_SUBMIT_FAILED": 123,
        "THREAD_POOL_TASK_TIMED_OUT": 124,
        "UNREACHABLE_IMPALADS": 125,
        "INACTIVE_SESSION_EXPIRED": 126,
        "INACTIVE_QUERY_EXPIRED": 127,
        "EXEC_TIME_LIMIT_EXCEEDED": 128,
        "CPU_LIMIT_EXCEEDED": 129,
        "SCAN_BYTES_LIMIT_EXCEEDED": 130,
        "ROWS_PRODUCED_LIMIT_EXCEEDED": 131,
        "EXPR_REWRITE_RESULT_LIMIT_EXCEEDED": 132,
        "UNRESPONSIVE_BACKEND": 133,
        "PARQUET_DATE_OUT_OF_RANGE": 134,
        "DISCONNECTED_SESSION_CLOSED": 135,
        "UNAUTHORIZED_SESSION_USER": 136,
        "ZSTD_ERROR": 137,
        "LZ4_BLOCK_DECOMPRESS_DECOMPRESS_SIZE_INCORRECT": 138,
        "LZ4_BLOCK_DECOMPRESS_INVALID_INPUT_LENGTH": 139,
        "LZ4_BLOCK_DECOMPRESS_INVALID_COMPRESSED_LENGTH": 140,
        "LZ4_DECOMPRESS_SAFE_FAILED": 141,
        "LZ4_COMPRESS_DEFAULT_FAILED": 142,
        "MAX_STATEMENT_LENGTH_EXCEEDED": 143,
        "AVRO_INVALID_DATE": 144,
        "ORC_TIMESTAMP_OUT_OF_RANGE": 145,
        "ORC_DATE_OUT_OF_RANGE": 146,
        "ORC_NESTED_TYPE_MISMATCH": 147,
        "ORC_TYPE_NOT_ROOT_AT_STRUCT": 148,
        "NAAJ_OUT_OF_MEMORY": 149,
        "INVALID_QUERY_HANDLE": 150,
        "JOIN_ROWS_PRODUCED_LIMIT_EXCEEDED": 151,
        "LOCAL_DISK_FAULTY": 152,
        "JWKS_PARSE_ERROR": 153,
        "JWT_VERIFY_FAILED": 154,
        "PARQUET_ROWS_SKIPPING": 155,
        "QUERY_OPTION_PARSE_FAILED": 156,
        "CATALOG_INCOMPATIBLE_PROTOCOL": 157,
        "STATESTORE_INCOMPATIBLE_PROTOCOL": 158,
        "JDBC_CONFIGURATION_ERROR": 159,
        "TUPLE_CACHE_INCONSISTENCY": 160,
        "OAUTH_VERIFY_FAILED": 161,
    }
fix_spec(all_structs)
del all_structs



#
# Autogenerated by Thrift Compiler (0.16.0)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py:new_style,no_utf8strings
#

from thrift.Thrift import TType, TMessageType, TFrozenDict, TException, TApplicationException
from thrift.protocol.TProtocol import TProtocolException
from thrift.TRecursive import fix_spec

import impala._thrift_gen.TCLIService.TCLIService
import logging
from .ttypes import *
from thrift.Thrift import TProcessor
from thrift.transport import TTransport
all_structs = []


class Iface(impala._thrift_gen.TCLIService.TCLIService.Iface):
    def GetExecSummary(self, req):
        """
        Parameters:
         - req

        """
        pass

    def GetRuntimeProfile(self, req):
        """
        Parameters:
         - req

        """
        pass

    def PingImpalaHS2Service(self, req):
        """
        Parameters:
         - req

        """
        pass

    def CloseImpalaOperation(self, req):
        """
        Parameters:
         - req

        """
        pass


class Client(impala._thrift_gen.TCLIService.TCLIService.Client, Iface):
    def __init__(self, iprot, oprot=None):
        impala._thrift_gen.TCLIService.TCLIService.Client.__init__(self, iprot, oprot)

    def GetExecSummary(self, req):
        """
        Parameters:
         - req

        """
        self.send_GetExecSummary(req)
        return self.recv_GetExecSummary()

    def send_GetExecSummary(self, req):
        self._oprot.writeMessageBegin('GetExecSummary', TMessageType.CALL, self._seqid)
        args = GetExecSummary_args()
        args.req = req
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_GetExecSummary(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = GetExecSummary_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.success is not None:
            return result.success
        raise TApplicationException(TApplicationException.MISSING_RESULT, "GetExecSummary failed: unknown result")

    def GetRuntimeProfile(self, req):
        """
        Parameters:
         - req

        """
        self.send_GetRuntimeProfile(req)
        return self.recv_GetRuntimeProfile()

    def send_GetRuntimeProfile(self, req):
        self._oprot.writeMessageBegin('GetRuntimeProfile', TMessageType.CALL, self._seqid)
        args = GetRuntimeProfile_args()
        args.req = req
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_GetRuntimeProfile(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = GetRuntimeProfile_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.success is not None:
            return result.success
        raise TApplicationException(TApplicationException.MISSING_RESULT, "GetRuntimeProfile failed: unknown result")

    def PingImpalaHS2Service(self, req):
        """
        Parameters:
         - req

        """
        self.send_PingImpalaHS2Service(req)
        return self.recv_PingImpalaHS2Service()

    def send_PingImpalaHS2Service(self, req):
        self._oprot.writeMessageBegin('PingImpalaHS2Service', TMessageType.CALL, self._seqid)
        args = PingImpalaHS2Service_args()
        args.req = req
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_PingImpalaHS2Service(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = PingImpalaHS2Service_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.success is not None:
            return result.success
        raise TApplicationException(TApplicationException.MISSING_RESULT, "PingImpalaHS2Service failed: unknown result")

    def CloseImpalaOperation(self, req):
        """
        Parameters:
         - req

        """
        self.send_CloseImpalaOperation(req)
        return self.recv_CloseImpalaOperation()

    def send_CloseImpalaOperation(self, req):
        self._oprot.writeMessageBegin('CloseImpalaOperation', TMessageType.CALL, self._seqid)
        args = CloseImpalaOperation_args()
        args.req = req
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_CloseImpalaOperation(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = CloseImpalaOperation_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.success is not None:
            return result.success
        raise TApplicationException(TApplicationException.MISSING_RESULT, "CloseImpalaOperation failed: unknown result")


class Processor(impala._thrift_gen.TCLIService.TCLIService.Processor, Iface, TProcessor):
    def __init__(self, handler):
        impala._thrift_gen.TCLIService.TCLIService.Processor.__init__(self, handler)
        self._processMap["GetExecSummary"] = Processor.process_GetExecSummary
        self._processMap["GetRuntimeProfile"] = Processor.process_GetRuntimeProfile
        self._processMap["PingImpalaHS2Service"] = Processor.process_PingImpalaHS2Service
        self._processMap["CloseImpalaOperation"] = Processor.process_CloseImpalaOperation
        self._on_message_begin = None

    def on_message_begin(self, func):
        self._on_message_begin = func

    def process(self, iprot, oprot):
        (name, type, seqid) = iprot.readMessageBegin()
        if self._on_message_begin:
            self._on_message_begin(name, type, seqid)
        if name not in self._processMap:
            iprot.skip(TType.STRUCT)
            iprot.readMessageEnd()
            x = TApplicationException(TApplicationException.UNKNOWN_METHOD, 'Unknown function %s' % (name))
            oprot.writeMessageBegin(name, TMessageType.EXCEPTION, seqid)
            x.write(oprot)
            oprot.writeMessageEnd()
            oprot.trans.flush()
            return
        else:
            self._processMap[name](self, seqid, iprot, oprot)
        return True

    def process_GetExecSummary(self, seqid, iprot, oprot):
        args = GetExecSummary_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = GetExecSummary_result()
        try:
            result.success = self._handler.GetExecSummary(args.req)
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("GetExecSummary", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def process_GetRuntimeProfile(self, seqid, iprot, oprot):
        args = GetRuntimeProfile_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = GetRuntimeProfile_result()
        try:
            result.success = self._handler.GetRuntimeProfile(args.req)
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("GetRuntimeProfile", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def process_PingImpalaHS2Service(self, seqid, iprot, oprot):
        args = PingImpalaHS2Service_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = PingImpalaHS2Service_result()
        try:
            result.success = self._handler.PingImpalaHS2Service(args.req)
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("PingImpalaHS2Service", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def process_CloseImpalaOperation(self, seqid, iprot, oprot):
        args = CloseImpalaOperation_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = CloseImpalaOperation_result()
        try:
            result.success = self._handler.CloseImpalaOperation(args.req)
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("CloseImpalaOperation", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

# HELPER FUNCTIONS AND STRUCTURES


class GetExecSummary_args(object):
    """
    Attributes:
     - req

    """


    def __init__(self, req=None,):
        self.req = req

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.req = TGetExecSummaryReq()
                    self.req.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('GetExecSummary_args')
        if self.req is not None:
            oprot.writeFieldBegin('req', TType.STRUCT, 1)
            self.req.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(GetExecSummary_args)
GetExecSummary_args.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'req', [TGetExecSummaryReq, None], None, ),  # 1
)


class GetExecSummary_result(object):
    """
    Attributes:
     - success

    """


    def __init__(self, success=None,):
        self.success = success

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 0:
                if ftype == TType.STRUCT:
                    self.success = TGetExecSummaryResp()
                    self.success.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('GetExecSummary_result')
        if self.success is not None:
            oprot.writeFieldBegin('success', TType.STRUCT, 0)
            self.success.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(GetExecSummary_result)
GetExecSummary_result.thrift_spec = (
    (0, TType.STRUCT, 'success', [TGetExecSummaryResp, None], None, ),  # 0
)


class GetRuntimeProfile_args(object):
    """
    Attributes:
     - req

    """


    def __init__(self, req=None,):
        self.req = req

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.req = TGetRuntimeProfileReq()
                    self.req.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('GetRuntimeProfile_args')
        if self.req is not None:
            oprot.writeFieldBegin('req', TType.STRUCT, 1)
            self.req.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(GetRuntimeProfile_args)
GetRuntimeProfile_args.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'req', [TGetRuntimeProfileReq, None], None, ),  # 1
)


class GetRuntimeProfile_result(object):
    """
    Attributes:
     - success

    """


    def __init__(self, success=None,):
        self.success = success

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 0:
                if ftype == TType.STRUCT:
                    self.success = TGetRuntimeProfileResp()
                    self.success.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('GetRuntimeProfile_result')
        if self.success is not None:
            oprot.writeFieldBegin('success', TType.STRUCT, 0)
            self.success.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(GetRuntimeProfile_result)
GetRuntimeProfile_result.thrift_spec = (
    (0, TType.STRUCT, 'success', [TGetRuntimeProfileResp, None], None, ),  # 0
)


class PingImpalaHS2Service_args(object):
    """
    Attributes:
     - req

    """


    def __init__(self, req=None,):
        self.req = req

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.req = TPingImpalaHS2ServiceReq()
                    self.req.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('PingImpalaHS2Service_args')
        if self.req is not None:
            oprot.writeFieldBegin('req', TType.STRUCT, 1)
            self.req.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(PingImpalaHS2Service_args)
PingImpalaHS2Service_args.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'req', [TPingImpalaHS2ServiceReq, None], None, ),  # 1
)


class PingImpalaHS2Service_result(object):
    """
    Attributes:
     - success

    """


    def __init__(self, success=None,):
        self.success = success

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 0:
                if ftype == TType.STRUCT:
                    self.success = TPingImpalaHS2ServiceResp()
                    self.success.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('PingImpalaHS2Service_result')
        if self.success is not None:
            oprot.writeFieldBegin('success', TType.STRUCT, 0)
            self.success.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(PingImpalaHS2Service_result)
PingImpalaHS2Service_result.thrift_spec = (
    (0, TType.STRUCT, 'success', [TPingImpalaHS2ServiceResp, None], None, ),  # 0
)


class CloseImpalaOperation_args(object):
    """
    Attributes:
     - req

    """


    def __init__(self, req=None,):
        self.req = req

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.req = TCloseImpalaOperationReq()
                    self.req.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('CloseImpalaOperation_args')
        if self.req is not None:
            oprot.writeFieldBegin('req', TType.STRUCT, 1)
            self.req.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(CloseImpalaOperation_args)
CloseImpalaOperation_args.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'req', [TCloseImpalaOperationReq, None], None, ),  # 1
)


class CloseImpalaOperation_result(object):
    """
    Attributes:
     - success

    """


    def __init__(self, success=None,):
        self.success = success

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 0:
                if ftype == TType.STRUCT:
                    self.success = TCloseImpalaOperationResp()
                    self.success.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('CloseImpalaOperation_result')
        if self.success is not None:
            oprot.writeFieldBegin('success', TType.STRUCT, 0)
            self.success.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(CloseImpalaOperation_result)
CloseImpalaOperation_result.thrift_spec = (
    (0, TType.STRUCT, 'success', [TCloseImpalaOperationResp, None], None, ),  # 0
)
fix_spec(all_structs)
del all_structs



#
# Autogenerated by Thrift Compiler (0.16.0)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py:new_style,no_utf8strings
#

from thrift.Thrift import TType, TMessageType, TFrozenDict, TException, TApplicationException
from thrift.protocol.TProtocol import TProtocolException
from thrift.TRecursive import fix_spec


from thrift.transport import TTransport
all_structs = []


class TUnit(object):
    UNIT = 0
    UNIT_PER_SECOND = 1
    CPU_TICKS = 2
    BYTES = 3
    BYTES_PER_SECOND = 4
    TIME_NS = 5
    DOUBLE_VALUE = 6
    NONE = 7
    TIME_MS = 8
    TIME_S = 9
    TIME_US = 10
    BASIS_POINTS = 11

    _VALUES_TO_NAMES = {
        0: "UNIT",
        1: "UNIT_PER_SECOND",
        2: "CPU_TICKS",
        3: "BYTES",
        4: "BYTES_PER_SECOND",
        5: "TIME_NS",
        6: "DOUBLE_VALUE",
        7: "NONE",
        8: "TIME_MS",
        9: "TIME_S",
        10: "TIME_US",
        11: "BASIS_POINTS",
    }

    _NAMES_TO_VALUES = {
        "UNIT": 0,
        "UNIT_PER_SECOND": 1,
        "CPU_TICKS": 2,
        "BYTES": 3,
        "BYTES_PER_SECOND": 4,
        "TIME_NS": 5,
        "DOUBLE_VALUE": 6,
        "NONE": 7,
        "TIME_MS": 8,
        "TIME_S": 9,
        "TIME_US": 10,
        "BASIS_POINTS": 11,
    }


class TMetricKind(object):
    GAUGE = 0
    COUNTER = 1
    PROPERTY = 2
    STATS = 3
    SET = 4
    HISTOGRAM = 5

    _VALUES_TO_NAMES = {
        0: "GAUGE",
        1: "COUNTER",
        2: "PROPERTY",
        3: "STATS",
        4: "SET",
        5: "HISTOGRAM",
    }

    _NAMES_TO_VALUES = {
        "GAUGE": 0,
        "COUNTER": 1,
        "PROPERTY": 2,
        "STATS": 3,
        "SET": 4,
        "HISTOGRAM": 5,
    }
fix_spec(all_structs)
del all_structs



#
# Autogenerated by Thrift Compiler (0.16.0)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py:new_style,no_utf8strings
#

from thrift.Thrift import TType, TMessageType, TFrozenDict, TException, TApplicationException
from thrift.protocol.TProtocol import TProtocolException
from thrift.TRecursive import fix_spec

import impala._thrift_gen.Status.ttypes
import impala._thrift_gen.Types.ttypes

from thrift.transport import TTransport
all_structs = []


class TExecState(object):
    REGISTERED = 0
    PLANNING = 1
    QUEUED = 2
    RUNNING = 3
    FINISHED = 4
    CANCELLED = 5
    FAILED = 6

    _VALUES_TO_NAMES = {
        0: "REGISTERED",
        1: "PLANNING",
        2: "QUEUED",
        3: "RUNNING",
        4: "FINISHED",
        5: "CANCELLED",
        6: "FAILED",
    }

    _NAMES_TO_VALUES = {
        "REGISTERED": 0,
        "PLANNING": 1,
        "QUEUED": 2,
        "RUNNING": 3,
        "FINISHED": 4,
        "CANCELLED": 5,
        "FAILED": 6,
    }


class TExecStats(object):
    """
    Attributes:
     - latency_ns
     - cpu_time_ns
     - cardinality
     - memory_used

    """


    def __init__(self, latency_ns=None, cpu_time_ns=None, cardinality=None, memory_used=None,):
        self.latency_ns = latency_ns
        self.cpu_time_ns = cpu_time_ns
        self.cardinality = cardinality
        self.memory_used = memory_used

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.I64:
                    self.latency_ns = iprot.readI64()
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.I64:
                    self.cpu_time_ns = iprot.readI64()
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.I64:
                    self.cardinality = iprot.readI64()
                else:
                    iprot.skip(ftype)
            elif fid == 4:
                if ftype == TType.I64:
                    self.memory_used = iprot.readI64()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('TExecStats')
        if self.latency_ns is not None:
            oprot.writeFieldBegin('latency_ns', TType.I64, 1)
            oprot.writeI64(self.latency_ns)
            oprot.writeFieldEnd()
        if self.cpu_time_ns is not None:
            oprot.writeFieldBegin('cpu_time_ns', TType.I64, 2)
            oprot.writeI64(self.cpu_time_ns)
            oprot.writeFieldEnd()
        if self.cardinality is not None:
            oprot.writeFieldBegin('cardinality', TType.I64, 3)
            oprot.writeI64(self.cardinality)
            oprot.writeFieldEnd()
        if self.memory_used is not None:
            oprot.writeFieldBegin('memory_used', TType.I64, 4)
            oprot.writeI64(self.memory_used)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class TPlanNodeExecSummary(object):
    """
    Attributes:
     - node_id
     - fragment_idx
     - label
     - label_detail
     - num_children
     - estimated_stats
     - exec_stats
     - is_broadcast
     - num_hosts

    """


    def __init__(self, node_id=None, fragment_idx=None, label=None, label_detail=None, num_children=None, estimated_stats=None, exec_stats=None, is_broadcast=None, num_hosts=None,):
        self.node_id = node_id
        self.fragment_idx = fragment_idx
        self.label = label
        self.label_detail = label_detail
        self.num_children = num_children
        self.estimated_stats = estimated_stats
        self.exec_stats = exec_stats
        self.is_broadcast = is_broadcast
        self.num_hosts = num_hosts

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.I32:
                    self.node_id = iprot.readI32()
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.I32:
                    self.fragment_idx = iprot.readI32()
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.STRING:
                    self.label = iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 4:
                if ftype == TType.STRING:
                    self.label_detail = iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 5:
                if ftype == TType.I32:
                    self.num_children = iprot.readI32()
                else:
                    iprot.skip(ftype)
            elif fid == 6:
                if ftype == TType.STRUCT:
                    self.estimated_stats = TExecStats()
                    self.estimated_stats.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 7:
                if ftype == TType.LIST:
                    self.exec_stats = []
                    (_etype3, _size0) = iprot.readListBegin()
                    for _i4 in range(_size0):
                        _elem5 = TExecStats()
                        _elem5.read(iprot)
                        self.exec_stats.append(_elem5)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 8:
                if ftype == TType.BOOL:
                    self.is_broadcast = iprot.readBool()
                else:
                    iprot.skip(ftype)
            elif fid == 9:
                if ftype == TType.I32:
                    self.num_hosts = iprot.readI32()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('TPlanNodeExecSummary')
        if self.node_id is not None:
            oprot.writeFieldBegin('node_id', TType.I32, 1)
            oprot.writeI32(self.node_id)
            oprot.writeFieldEnd()
        if self.fragment_idx is not None:
            oprot.writeFieldBegin('fragment_idx', TType.I32, 2)
            oprot.writeI32(self.fragment_idx)
            oprot.writeFieldEnd()
        if self.label is not None:
            oprot.writeFieldBegin('label', TType.STRING, 3)
            oprot.writeString(self.label)
            oprot.writeFieldEnd()
        if self.label_detail is not None:
            oprot.writeFieldBegin('label_detail', TType.STRING, 4)
            oprot.writeString(self.label_detail)
            oprot.writeFieldEnd()
        if self.num_children is not None:
            oprot.writeFieldBegin('num_children', TType.I32, 5)
            oprot.writeI32(self.num_children)
            oprot.writeFieldEnd()
        if self.estimated_stats is not None:
            oprot.writeFieldBegin('estimated_stats', TType.STRUCT, 6)
            self.estimated_stats.write(oprot)
            oprot.writeFieldEnd()
        if self.exec_stats is not None:
            oprot.writeFieldBegin('exec_stats', TType.LIST, 7)
            oprot.writeListBegin(TType.STRUCT, len(self.exec_stats))
            for iter6 in self.exec_stats:
                iter6.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.is_broadcast is not None:
            oprot.writeFieldBegin('is_broadcast', TType.BOOL, 8)
            oprot.writeBool(self.is_broadcast)
            oprot.writeFieldEnd()
        if self.num_hosts is not None:
            oprot.writeFieldBegin('num_hosts', TType.I32, 9)
            oprot.writeI32(self.num_hosts)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        if self.node_id is None:
            raise TProtocolException(message='Required field node_id is unset!')
        if self.fragment_idx is None:
            raise TProtocolException(message='Required field fragment_idx is unset!')
        if self.label is None:
            raise TProtocolException(message='Required field label is unset!')
        if self.num_children is None:
            raise TProtocolException(message='Required field num_children is unset!')
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class TExecProgress(object):
    """
    Attributes:
     - total_scan_ranges
     - num_completed_scan_ranges
     - total_fragment_instances
     - num_completed_fragment_instances

    """


    def __init__(self, total_scan_ranges=None, num_completed_scan_ranges=None, total_fragment_instances=None, num_completed_fragment_instances=None,):
        self.total_scan_ranges = total_scan_ranges
        self.num_completed_scan_ranges = num_completed_scan_ranges
        self.total_fragment_instances = total_fragment_instances
        self.num_completed_fragment_instances = num_completed_fragment_instances

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.I64:
                    self.total_scan_ranges = iprot.readI64()
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.I64:
                    self.num_completed_scan_ranges = iprot.readI64()
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.I64:
                    self.total_fragment_instances = iprot.readI64()
                else:
                    iprot.skip(ftype)
            elif fid == 4:
                if ftype == TType.I64:
                    self.num_completed_fragment_instances = iprot.readI64()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('TExecProgress')
        if self.total_scan_ranges is not None:
            oprot.writeFieldBegin('total_scan_ranges', TType.I64, 1)
            oprot.writeI64(self.total_scan_ranges)
            oprot.writeFieldEnd()
        if self.num_completed_scan_ranges is not None:
            oprot.writeFieldBegin('num_completed_scan_ranges', TType.I64, 2)
            oprot.writeI64(self.num_completed_scan_ranges)
            oprot.writeFieldEnd()
        if self.total_fragment_instances is not None:
            oprot.writeFieldBegin('total_fragment_instances', TType.I64, 3)
            oprot.writeI64(self.total_fragment_instances)
            oprot.writeFieldEnd()
        if self.num_completed_fragment_instances is not None:
            oprot.writeFieldBegin('num_completed_fragment_instances', TType.I64, 4)
            oprot.writeI64(self.num_completed_fragment_instances)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class TExecSummary(object):
    """
    Attributes:
     - state
     - status
     - nodes
     - exch_to_sender_map
     - error_logs
     - progress
     - is_queued
     - queued_reason

    """


    def __init__(self, state=None, status=None, nodes=None, exch_to_sender_map=None, error_logs=None, progress=None, is_queued=None, queued_reason=None,):
        self.state = state
        self.status = status
        self.nodes = nodes
        self.exch_to_sender_map = exch_to_sender_map
        self.error_logs = error_logs
        self.progress = progress
        self.is_queued = is_queued
        self.queued_reason = queued_reason

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.I32:
                    self.state = iprot.readI32()
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRUCT:
                    self.status = impala._thrift_gen.Status.ttypes.TStatus()
                    self.status.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.LIST:
                    self.nodes = []
                    (_etype10, _size7) = iprot.readListBegin()
                    for _i11 in range(_size7):
                        _elem12 = TPlanNodeExecSummary()
                        _elem12.read(iprot)
                        self.nodes.append(_elem12)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 4:
                if ftype == TType.MAP:
                    self.exch_to_sender_map = {}
                    (_ktype14, _vtype15, _size13) = iprot.readMapBegin()
                    for _i17 in range(_size13):
                        _key18 = iprot.readI32()
                        _val19 = iprot.readI32()
                        self.exch_to_sender_map[_key18] = _val19
                    iprot.readMapEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 5:
                if ftype == TType.LIST:
                    self.error_logs = []
                    (_etype23, _size20) = iprot.readListBegin()
                    for _i24 in range(_size20):
                        _elem25 = iprot.readString()
                        self.error_logs.append(_elem25)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 6:
                if ftype == TType.STRUCT:
                    self.progress = TExecProgress()
                    self.progress.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 7:
                if ftype == TType.BOOL:
                    self.is_queued = iprot.readBool()
                else:
                    iprot.skip(ftype)
            elif fid == 8:
                if ftype == TType.STRING:
                    self.queued_reason = iprot.readString()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('TExecSummary')
        if self.state is not None:
            oprot.writeFieldBegin('state', TType.I32, 1)
            oprot.writeI32(self.state)
            oprot.writeFieldEnd()
        if self.status is not None:
            oprot.writeFieldBegin('status', TType.STRUCT, 2)
            self.status.write(oprot)
            oprot.writeFieldEnd()
        if self.nodes is not None:
            oprot.writeFieldBegin('nodes', TType.LIST, 3)
            oprot.writeListBegin(TType.STRUCT, len(self.nodes))
            for iter26 in self.nodes:
                iter26.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.exch_to_sender_map is not None:
            oprot.writeFieldBegin('exch_to_sender_map', TType.MAP, 4)
            oprot.writeMapBegin(TType.I32, TType.I32, len(self.exch_to_sender_map))
            for kiter27, viter28 in self.exch_to_sender_map.items():
                oprot.writeI32(kiter27)
                oprot.writeI32(viter28)
            oprot.writeMapEnd()
            oprot.writeFieldEnd()
        if self.error_logs is not None:
            oprot.writeFieldBegin('error_logs', TType.LIST, 5)
            oprot.writeListBegin(TType.STRING, len(self.error_logs))
            for iter29 in self.error_logs:
                oprot.writeString(iter29)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.progress is not None:
            oprot.writeFieldBegin('progress', TType.STRUCT, 6)
            self.progress.write(oprot)
            oprot.writeFieldEnd()
        if self.is_queued is not None:
            oprot.writeFieldBegin('is_queued', TType.BOOL, 7)
            oprot.writeBool(self.is_queued)
            oprot.writeFieldEnd()
        if self.queued_reason is not None:
            oprot.writeFieldBegin('queued_reason', TType.STRING, 8)
            oprot.writeString(self.queued_reason)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        if self.state is None:
            raise TProtocolException(message='Required field state is unset!')
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(TExecStats)
TExecStats.thrift_spec = (
    None,  # 0
    (1, TType.I64, 'latency_ns', None, None, ),  # 1
    (2, TType.I64, 'cpu_time_ns', None, None, ),  # 2
    (3, TType.I64, 'cardinality', None, None, ),  # 3
    (4, TType.I64, 'memory_used', None, None, ),  # 4
)
all_structs.append(TPlanNodeExecSummary)
TPlanNodeExecSummary.thrift_spec = (
    None,  # 0
    (1, TType.I32, 'node_id', None, None, ),  # 1
    (2, TType.I32, 'fragment_idx', None, None, ),  # 2
    (3, TType.STRING, 'label', None, None, ),  # 3
    (4, TType.STRING, 'label_detail', None, None, ),  # 4
    (5, TType.I32, 'num_children', None, None, ),  # 5
    (6, TType.STRUCT, 'estimated_stats', [TExecStats, None], None, ),  # 6
    (7, TType.LIST, 'exec_stats', (TType.STRUCT, [TExecStats, None], False), None, ),  # 7
    (8, TType.BOOL, 'is_broadcast', None, None, ),  # 8
    (9, TType.I32, 'num_hosts', None, None, ),  # 9
)
all_structs.append(TExecProgress)
TExecProgress.thrift_spec = (
    None,  # 0
    (1, TType.I64, 'total_scan_ranges', None, None, ),  # 1
    (2, TType.I64, 'num_completed_scan_ranges', None, None, ),  # 2
    (3, TType.I64, 'total_fragment_instances', None, None, ),  # 3
    (4, TType.I64, 'num_completed_fragment_instances', None, None, ),  # 4
)
all_structs.append(TExecSummary)
TExecSummary.thrift_spec = (
    None,  # 0
    (1, TType.I32, 'state', None, None, ),  # 1
    (2, TType.STRUCT, 'status', [impala._thrift_gen.Status.ttypes.TStatus, None], None, ),  # 2
    (3, TType.LIST, 'nodes', (TType.STRUCT, [TPlanNodeExecSummary, None], False), None, ),  # 3
    (4, TType.MAP, 'exch_to_sender_map', (TType.I32, None, TType.I32, None, False), None, ),  # 4
    (5, TType.LIST, 'error_logs', (TType.STRING, None, False), None, ),  # 5
    (6, TType.STRUCT, 'progress', [TExecProgress, None], None, ),  # 6
    (7, TType.BOOL, 'is_queued', None, None, ),  # 7
    (8, TType.STRING, 'queued_reason', None, None, ),  # 8
)
fix_spec(all_structs)
del all_structs



# Copyright 2014 Cloudera Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

import datetime
import re
import six
from six import reraise

from impala.util import _escape
from impala.error import (  # pylint: disable=unused-import
    Error, Warning, InterfaceError, DatabaseError, InternalError,
    OperationalError, ProgrammingError, IntegrityError, DataError,
    NotSupportedError)


class Connection(object):
    # PEP 249
    # Connection objects are associated with a TCLIService.Client thrift
    # service
    # it's instantiated with an alive TCLIService.Client

    def close(self):
        # PEP 249
        raise NotImplementedError

    def commit(self):
        # PEP 249
        raise NotImplementedError

    def rollback(self):
        # PEP 249
        raise NotImplementedError

    def cursor(self, user=None, configuration=None, convert_types=True):
        # PEP 249
        raise NotImplementedError

    def reconnect(self):
        raise NotImplementedError

    def kerberized(self):
        # pylint: disable=protected-access
        # returns bool whether underlying service is kerberized or not
        from thrift_sasl import TSaslClientTransport
        if isinstance(self.service._iprot.trans, TSaslClientTransport):
            if self.service._iprot.trans.mechanism == 'GSSAPI':
                return True
        return False

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()
        if exc_type is not None:
            reraise(exc_type, exc_val, exc_tb)

    # optional DB API addition to make the errors attributes of Connection
    Error = Error
    Warning = Warning
    InterfaceError = InterfaceError
    DatabaseError = DatabaseError
    InternalError = InternalError
    OperationalError = OperationalError
    ProgrammingError = ProgrammingError
    IntegrityError = IntegrityError
    DataError = DataError
    NotSupportedError = NotSupportedError


class Cursor(object):
    """Abstract representation of Cursor"""

    def description(self):
        raise NotImplementedError

    def rowcount(self):
        raise NotImplementedError

    def lastrowid(self):
        raise NotImplementedError

    def query_string(self):
        raise NotImplementedError

    def get_arraysize(self):
        raise NotImplementedError

    def set_arraysize(self, arraysize):
        raise NotImplementedError

    def buffersize(self):
        raise NotImplementedError

    def has_result_set(self):
        raise NotImplementedError

    def close(self):
        raise NotImplementedError

    def close_operation(self):
        raise NotImplementedError

    def execute(self, operation, parameters=None, configuration=None):
        raise NotImplementedError

    def _execute_sync(self, operation_fn):
        raise NotImplementedError

    def _reset_state(self):
        raise NotImplementedError

    def _wait_to_finish(self):
        raise NotImplementedError

    def executemany(self, operation, seq_of_parameters):
        raise NotImplementedError

    def fetchone(self):
        raise NotImplementedError

    def fetchmany(self, size=None):
        raise NotImplementedError

    def fetchall(self):
        raise NotImplementedError

    def setinputsizes(self, sizes):
        raise NotImplementedError

    def setoutputsize(self, size, column=None):
        raise NotImplementedError

    def __iter__(self):
        raise NotImplementedError

    def __next__(self):
        raise NotImplementedError

    def next(self):
        # for py2 compat
        return self.__next__()

    def ping(self):
        raise NotImplementedError

    def get_log(self):
        raise NotImplementedError

    def get_profile(self):
        raise NotImplementedError

    def get_summary(self):
        raise NotImplementedError

    def build_summary_table(self, summary, idx, is_fragment_root, indent_level,
                            output):
        raise NotImplementedError

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()
        if exc_type is not None:
            reraise(exc_type, exc_val, exc_tb)


def _replace_numeric_markers(operation, string_parameters, paramstyle):
    """
    Replaces qmark, format, and numeric markers in the given operation, from
    the string_parameters list.

    Raises ProgrammingError on wrong number of parameters or markers.
    For numeric markers there has to be enough parameters to satisfy
    every marker and there has to bo no unused parameter.
    """
    RE_QMARK = r'(?P<qmark>\?)'
    RE_FORMAT = r'(?P<format>%s)'
    RE_NUMERIC = r'(?P<numeric>:(?P<index>\d+))'
    RE_ALL = '|'.join([RE_QMARK, RE_FORMAT, RE_NUMERIC])

    if paramstyle is not None:
        if paramstyle in ['named', 'pyformat']:
            raise ProgrammingError(
                "Paramstyle '%s' is not compatible with parameters passed as "
                "list. Please use a dict for your parameters instead "
                "or specify a different paramstyle" % paramstyle)

        if paramstyle not in ['qmark', 'format', 'numeric']:
            raise ProgrammingError(
                "Paramstyle '%s' is not supported. Please use a different one")

    param_count = len(string_parameters)
    used_positional_indexes = set()
    used_numeric_indexes = set()

    def replace_marker(match):
        if paramstyle is not None and match.group(paramstyle) is None:
            return match.group(0)

        if match.group('index') is not None:
            param_index = int(match.group('index')) - 1
            used_numeric_indexes.add(param_index)
        else:
            param_index = len(used_positional_indexes)
            used_positional_indexes.add(param_index)

        if param_index >= param_count:
            raise ProgrammingError(
                "Incorrect number of bindings supplied. The current statement "
                "uses %d or more, and there are %d supplied." % (
                    param_index, param_count))

        return string_parameters[param_index]

    operation = re.sub(RE_ALL, replace_marker, operation)

    marker_count = len(used_numeric_indexes | used_positional_indexes)
    if marker_count < param_count:
        raise ProgrammingError(
            "Incorrect number of bindings supplied. The current statement "
            "uses %d, and there are %d supplied." % (
                marker_count, param_count))

    return operation


def _bind_parameters_list(operation, parameters, paramstyle):
    string_parameters = []
    for value in parameters:
        if value is None:
            string_parameters.append('NULL')
        elif isinstance(value, six.string_types):
            string_parameters.append("'" + _escape(value) + "'")
        elif isinstance(value, datetime.datetime):
            string_parameters.append("'" + str(value) + "'")
        elif isinstance(value, datetime.date):
            string_parameters.append("'" + str(value) + "'")
        else:
            string_parameters.append(str(value))

    # replace qmark and numeric parameters
    return _replace_numeric_markers(operation, string_parameters, paramstyle)


def _bind_parameters_dict(operation, parameters):
    string_parameters = {}
    for (name, value) in six.iteritems(parameters):
        if value is None:
            string_parameters[name] = 'NULL'
        elif isinstance(value, six.string_types):
            string_parameters[name] = "'" + _escape(value) + "'"
        elif isinstance(value, datetime.date):
            string_parameters[name] = "'{0}'".format(value)
        else:
            string_parameters[name] = str(value)

    # replace named parameters by their pyformat equivalents
    operation = re.sub(r":([^\d\W]\w*)", r"%(\g<1>)s", operation)

    # replace pyformat parameters
    return operation % string_parameters


def _bind_parameters(operation, parameters, paramstyle=None):
    # If parameters is a list, assume either qmark, format, or numeric
    # format. If not, assume either named or pyformat parameters
    if isinstance(parameters, (list, tuple)):
        return _bind_parameters_list(operation, parameters, paramstyle)
    elif isinstance(parameters, dict):
        return _bind_parameters_dict(operation, parameters)
    else:
        raise ProgrammingError("Query parameters argument should be a "
                               "list, tuple, or dict object")



# Copyright 2013 Cloudera Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

import ez_setup
ez_setup.use_setuptools()

from setuptools import setup, find_packages


def readme():
    with open('README.md', 'r') as ip:
        return ip.read()

setup(
    name='impyla',
    # impala/__init__.py also contains the version - the two should have the same value!
    version='v0.21.0',
    description='Python client for the Impala distributed query engine',
    long_description_content_type='text/markdown',
    long_description=readme(),
    maintainer='Wes McKinney',
    maintainer_email='wes.mckinney@twosigma.com',
    author='Uri Laserson',
    author_email='laserson@cloudera.com',
    url='https://github.com/cloudera/impyla',
    packages=find_packages(),
    install_package_data=True,
    package_data={'impala.thrift': ['*.thrift']},
    install_requires=['six', 'bitarray<3', 'thrift==0.16.0', 'thrift_sasl==0.4.3'],
    extras_require={
        "kerberos": ['kerberos>=1.3.0'],
    },
    keywords=('cloudera impala python hadoop sql hdfs mpp spark pydata '
              'pandas distributed db api pep 249 hive hiveserver2 hs2'),
    license='Apache License, Version 2.0',
    classifiers=[
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Programming Language :: Python :: 3.13',
    ],
    entry_points={
        'sqlalchemy.dialects': ['impala = impala.sqlalchemy:ImpalaDialect',
                                'impala4 = impala.sqlalchemy:Impala4Dialect']
    },
    zip_safe=False)



# Copyright 2013 Cloudera Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Implements the Python DB API 2.0 (PEP 249) for Impala"""

from __future__ import absolute_import

import six
import time
import datetime

from impala.error import (  # noqa
    Error, Warning, InterfaceError, DatabaseError, InternalError,
    OperationalError, ProgrammingError, IntegrityError, DataError,
    NotSupportedError)
from impala.util import (
    warn_deprecate, warn_protocol_param, warn_nontls_jwt)
import impala.hiveserver2 as hs2


AUTH_MECHANISMS = ['NOSASL', 'PLAIN', 'GSSAPI', 'LDAP', 'JWT']


# PEP 249 module globals
apilevel = '2.0'
threadsafety = 1  # Threads may share the module, but not connections
paramstyle = 'pyformat'


def connect(host='localhost', port=21050, database=None, timeout=None,
            use_ssl=False, ca_cert=None, auth_mechanism='NOSASL', user=None,
            password=None, kerberos_service_name='impala', use_ldap=None,
            ldap_user=None, ldap_password=None, use_kerberos=None,
            protocol=None, krb_host=None, use_http_transport=False,
            http_path='', auth_cookie_names=None, http_cookie_names=None,
            retries=3, jwt=None, user_agent=None,
            get_user_custom_headers_func=None):
    """Get a connection to HiveServer2 (HS2).

    These options are largely compatible with the impala-shell command line
    arguments. See those docs for more information.

    Parameters
    ----------
    host : str
        The hostname for HS2. For Impala, this can be any of the `impalad`s.
    port : int, optional
        The port number for HS2. The Impala default is 21050. The Hive port is
        likely different.
    database : str, optional
        The default database. If `None`, the result is
        implementation-dependent.
    timeout : int, optional
        Connection timeout in seconds. Default is no timeout.
    use_ssl : bool, optional
        Enable SSL.
    ca_cert : str, optional
        Local path to the the third-party CA certificate. If SSL is enabled but
        the certificate is not specified, the server certificate will not be
        validated.
    auth_mechanism : {'NOSASL', 'PLAIN', 'GSSAPI', 'LDAP', 'JWT'}
        Specify the authentication mechanism. `'NOSASL'` for unsecured Impala.
        `'PLAIN'` for unsecured Hive (because Hive requires the SASL
        transport). `'GSSAPI'` for Kerberos and `'LDAP'` for Kerberos with
        LDAP. `'JWT'` requires providing a JSON Web Token via the jwt parameter
        and only works with use_http_transport=True.
    user : str, optional
        LDAP user, if applicable.
    password : str, optional
        LDAP password, if applicable.
    kerberos_service_name : str, optional
        Authenticate to a particular `impalad` service principal. Uses
        `'impala'` by default.
    use_http_transport: bool optional
        Set it to True to use http transport of False to use binary transport.
    http_path: str, optional
        Specify the path in the http URL. Used only when `use_http_transport` is True.
    http_cookie_names: list of str or str, optional
        Specify the list of possible names for the cookies used for cookie-based
        authentication or session management. If the list of names contains one cookie
        name only, a str value can be specified instead of a list.
        If a cookie with one of these names is returned in an http response by the server
        or an intermediate proxy then it will be included in each subsequent request for
        the same connection. If set to wildcard ('*'), all cookies in an http response
        will be preserved. By default 'http_cookie_names' is set to '*'.
        Used only when `use_http_transport` is True.
        The names of authentication cookies are expected to end with ".auth" string, for
        example, "impala.auth" for Impala authentication cookies.
        If 'http_cookie_names' is explicitly set to a not None empty value ([], or ''),
        Impyla won't attempt to do cookie based authentication or session management.
        Currently cookie retention is supported for GSSAPI/LDAP/SASL/NOSASL/JWT over http.
    jwt: string containing a JSON Web Token
        This is used for auth_mechanism=JWT when using the HTTP transport.
    user_agent: A user specified user agent when HTTP transport is used. If none is specified,
        'Python/ImpylaHttpClient' is used
    use_ldap : bool, optional
        Specify `auth_mechanism='LDAP'` instead.
    get_user_custom_headers_func : function, optional
        Used to add custom headers to the http messages when using hs2-http protocol.
        This is a function returning a list of tuples, each tuple contains a key-value
        pair. This allows duplicate headers to be set.

        .. deprecated:: 0.18.0
    auth_cookie_names : list of str or str, optional
        Use `http_cookie_names` parameter instead.

        .. deprecated:: 0.18.0

        .. deprecated:: 0.11.0
    ldap_user : str, optional
        Use `user` parameter instead.

        .. deprecated:: 0.11.0
    ldap_password : str, optional
        Use `password` parameter instead.

        .. deprecated:: 0.11.0
    use_kerberos : bool, optional
        Specify `auth_mechanism='GSSAPI'` instead.

        .. deprecated:: 0.11.0
    protocol : str, optional
        Do not use.  HiveServer2 is the only protocol currently supported.

        .. deprecated:: 0.11.0


    Returns
    -------
    HiveServer2Connection
        A `Connection` object (DB API 2.0-compliant).
    """
    # pylint: disable=too-many-locals
    if use_kerberos is not None:
        warn_deprecate('use_kerberos', 'auth_mechanism="GSSAPI"')
        if use_kerberos:
            auth_mechanism = 'GSSAPI'

    if use_ldap is not None:
        warn_deprecate('use_ldap', 'auth_mechanism="LDAP"')
        if use_ldap:
            auth_mechanism = 'LDAP'

    if auth_mechanism:
        auth_mechanism = auth_mechanism.upper()
    else:
        auth_mechanism = 'NOSASL'

    if auth_mechanism not in AUTH_MECHANISMS:
        raise NotSupportedError(
            'Unsupported authentication mechanism: {0}'.format(auth_mechanism))

    if auth_mechanism == 'JWT':
        if jwt is None:
            raise NotSupportedError("JWT authentication requires specifying the 'jwt' argument")
        if not use_http_transport:
            raise NotSupportedError('JWT authentication is only supported for HTTP transport')
        if not use_ssl:
            warn_nontls_jwt()
        if user is not None or ldap_user is not None:
            raise NotSupportedError("'user' argument cannot be specified with '{0}' authentication".format(auth_mechanism))
        if password is not None or ldap_password is not None:
            raise NotSupportedError("'password' argument cannot be specified with '{0}' authentication".format(auth_mechanism))
    else:
        if jwt is not None:
            raise NotSupportedError("'jwt' argument cannot be specified with '{0}' authentication".format(auth_mechanism))

    if ldap_user is not None:
        warn_deprecate('ldap_user', 'user')
        user = ldap_user

    if ldap_password is not None:
        warn_deprecate('ldap_password', 'password')
        password = ldap_password

    if protocol is not None:
        if protocol.lower() == 'hiveserver2':
            warn_protocol_param()
        else:
            raise NotSupportedError(
                "'{0}' is not a supported protocol; only HiveServer2 is "
                "supported".format(protocol))

    if auth_cookie_names is not None and http_cookie_names is None:
        warn_deprecate('auth_cookie_names', 'http_cookie_names')
        http_cookie_names = auth_cookie_names
    elif http_cookie_names is None:
        # Preserve all cookies.
        http_cookie_names = '*'

    service = hs2.connect(host=host, port=port,
                          timeout=timeout, use_ssl=use_ssl,
                          ca_cert=ca_cert, user=user, password=password,
                          kerberos_service_name=kerberos_service_name,
                          auth_mechanism=auth_mechanism, krb_host=krb_host,
                          use_http_transport=use_http_transport,
                          http_path=http_path,
                          http_cookie_names=http_cookie_names,
                          retries=retries,
                          jwt=jwt, user_agent=user_agent,
                          get_user_custom_headers_func=get_user_custom_headers_func)
    return hs2.HiveServer2Connection(service, default_db=database)


class _DBAPITypeObject(object):
    # Compliance with Type Objects of PEP 249.

    def __init__(self, *values):
        self.values = values

    def __cmp__(self, other):
        if other in self.values:
            return 0
        else:
            return -1

    def __eq__(self, other):
        # py3 ignores __cmp__
        return other in self.values


STRING = _DBAPITypeObject('STRING')
BINARY = _DBAPITypeObject('BINARY')
NUMBER = _DBAPITypeObject('BOOLEAN', 'TINYINT', 'SMALLINT', 'INT', 'BIGINT',
                          'FLOAT', 'DOUBLE', 'DECIMAL')
DATETIME = _DBAPITypeObject('TIMESTAMP')
DATE = _DBAPITypeObject('DATE')
ROWID = _DBAPITypeObject()

Date = datetime.date
Time = datetime.time
Timestamp = datetime.datetime


def DateFromTicks(ticks):
    return Date(*time.localtime(ticks)[:3])


def TimeFromTicks(ticks):
    return Time(*time.localtime(ticks)[3:6])


def TimestampFromTicks(ticks):
    return Timestamp(*time.localtime(ticks)[:6])

if six.PY3:
    buffer = memoryview
Binary = buffer
